import logging.config
import platform
import subprocess
from pathlib import Path
from typing import List
from typing import Optional
from typing import Union

from fzf.constants import ENCODING
from fzf.constants import MULTI_FLAG
from fzf.logging_ import logging_config
from fzf.utilities import resolve_input


PathLike = Union[bytes, str, Path]


logging.config.dictConfig(logging_config)
logger = logging.getLogger(__name__)


class FZF:
    """Class abstracting FZF command."""

    def __init__(
        self,
        executable: str = None,
        input: Optional[str] = None,
        cwd: PathLike = None,
        multi: bool = False,
        fzf_extras: List[str] = None,
    ) -> None:

        if executable is None:
            executable = 'fzf'
        if fzf_extras is None:
            fzf_extras = []

        self.fzf = executable
        self.input = input
        self.cwd: Optional[PathLike] = cwd
        self.multi: bool = multi

        # Extra Arguments
        self.fzf_args: List = fzf_extras

    def run(self, *args, **kwargs) -> Union[str, List[str]]:
        """Given current configuration, run fzf and return selection."""

        # Buid Command
        command: List[str] = [self.fzf]
        if self.multi:
            command.append(MULTI_FLAG)

        command += self.fzf_args

        input_: bytes = resolve_input(self.input)
        if input_:
            kwargs['stdin'] = subprocess.PIPE

        # Run FZF as Process
        process: subprocess.Popen = subprocess.Popen(
            command,
            *args,
            stdout=subprocess.PIPE,
            encoding=ENCODING,
            cwd=self.cwd,
            **kwargs,
        )  # type: ignore

        # Process - Input
        if input_:
            stdout, _ = process.communicate(input_.decode(ENCODING))
        else:
            stdout, _ = process.communicate()

        # Process - Output
        if self.multi:
            return stdout.splitlines()
        else:
            return stdout.strip()

    @classmethod
    def install(cls, method=None, dry_run: bool = False) -> None:
        """Install fzf, using git.

        Will attempt to install FZF, if possible, matching the style of
        `fzf#install()` from the Vim plugin installation.

        [Documentation](https://github.com/junegunn/fzf#using-git)
        """
        cls.download(method=method)

        if dry_run:
            return
        else:
            cls._run_installer()

    @classmethod
    def _run_installer(cls) -> None:
        """Run fzf installer."""

        download_dir: Path = Path('./fzf')
        process_options: dict = {}

        if platform == 'Windows':
            install_script: Path = download_dir / 'install.ps1'
        else:
            install_script = download_dir / 'install'

        subprocess.run(['powershell.exe', install_script], **process_options)

    @classmethod
    def download(cls, method: str = None) -> None:

        if method is None:
            method = 'https'
        method = method.lower()

        options: List[str] = [
            *('--depth', '1'),
        ]

        if method == 'https' or method == 'http':
            exe: str = 'git'
            args: list[str] = [
                'clone',
            ]
            uri: str = 'https://github.com/junegunn/fzf.git'
        elif method == 'ssh':
            exe = 'git'
            args = [
                'clone',
            ]
            uri = 'git@github.com:junegunn/fzf.git'
        elif method == 'gh':
            exe = 'gh'
            args = [
                'repo',
                'clone',
            ]
            options = []
            uri = 'junegunn/fzf'

        command: list[str] = [exe] + args + options + [uri]

        subprocess.run(command)



import sys

if not sys.version_info < (3, 8):
    from typing import Final
from typing import Iterable
from typing import Union


if not sys.version_info < (3, 8):
    ENCODING: Final[str] = 'utf-8'
    MULTI_FLAG: Final[str] = '--multi'
else:
    ENCODING: str = 'utf-8'
    MULTI_FLAG: str = '--multi'
FZFInputValues = Union[bytes, str, Iterable[str], Iterable[bytes]]



import logging.config
from pathlib import Path


logs_path: Path = Path(__file__).parent / 'logs'
logs_path.mkdir(exist_ok=True)
log_file: Path = logs_path / Path(__file__).with_suffix('.log').name


logging_config: dict = dict(
    version=1,
    disable_existing_loggers=False,
    root=dict(
        level=logging.DEBUG,
        handlers=[
            'stream_handler',
            'file_handler',
        ],
    ),
    formatters={
        'default': {
            'date_format': '%Y-%m-%dT%H:%M:%S+0000%z',
            'format': '{asctime} - {name} - {levelname:<8} - {message}',
            'style': '{',
            'validate': True,
        },
    },
    handlers={
        'stream_handler': {
            'level': logging.DEBUG,
            'class': 'logging.StreamHandler',
            'formatter': 'default',
        },
        'file_handler': {
            'level': logging.DEBUG,
            'class': 'logging.FileHandler',
            'formatter': 'default',
            'filename': log_file,
        },
    },
    loggers={},
)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
import setuptools


with open('README.md', 'r') as fin:
    long_description = fin.read()
with open('LICENSE', 'r') as fin:
    license = fin.read()

setuptools.setup(
    name='fzflib',
    version='0.1.3',
    license='MIT',
    author='Kyle L. Davis',
    author_email='AceofSpades5757.github@gmail.com',
    url='https://github.com/AceofSpades5757/fzflib',
    description='A Python library for interacting with FZF.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    python_requires='>=3.6',
    packages=setuptools.find_packages('src'),
    package_dir={
        '': 'src',
        'fzflib': 'src/fzflib',
    },
    classifiers=[
        'Programming Language :: Python :: 3',
        'Operating System :: OS Independent',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
    ],
)
